--[[
	HAIBUTSU EDITA
	
	
	
	LIST OF ALL THE KEYS YOU NEED TO KNOW:
		B: ADD BLOCKS
		WASD: MOVE CAMERA
		1: CAMERA SCALE 1x
		2: CAMERA SCALE 2x
		3: CAMERA SCALE 0.5x
		T: CHANGE QUAD BY -1
		Y: CHANGE QUAD BY +1
	
	
	LIST OF ALL THE MODES:
		0: Startup
		1: Block Editing
		2: Idle
	
	
	
--]]

require "class"
require "camera"
require "dummy"
require "cursor"

function love.load()
	status_text = ""
	mode = 0
	build = 0101
	exports = 0
	math.randomseed(os.time())
	love.graphics.setMode(800,600)
	love.graphics.setCaption("Haibutsu 2013 Souken no Edita")
	love.filesystem.setIdentity("HaibutsuEdita")
	tile_grid = love.graphics.newImage("graphics/tilegrid.gif")
	edit_cursor = love.graphics.newImage("graphics/cursor.gif")
	

	
	tilesets = {
		love.graphics.newImage("graphics/TestZone.png"),
		love.graphics.newImage("graphics/Kinzo.png"),
	}
	
	blocks = {} -- holds all of the dummy blocks to draw
	
	quads = {
		-- quads for the test zone tileset
		love.graphics.newQuad(0,0,16,16,64,64),
		love.graphics.newQuad(16,0,16,16,64,64),
		love.graphics.newQuad(32,0,16,16,64,64),
		love.graphics.newQuad(48,0,16,16,64,64),
		love.graphics.newQuad(0,16,16,16,64,64),
		love.graphics.newQuad(16,16,16,16,64,64),
		love.graphics.newQuad(32,16,16,16,64,64),
		love.graphics.newQuad(48,16,16,16,64,64),
		love.graphics.newQuad(0,32,16,16,64,64),
		love.graphics.newQuad(16,32,16,16,64,64),
		love.graphics.newQuad(32,32,16,16,64,64),
		love.graphics.newQuad(48,32,16,16,64,64),
		love.graphics.newQuad(0,48,16,16,64,64),
		love.graphics.newQuad(16,48,16,16,64,64),
		love.graphics.newQuad(32,48,16,16,64,64),
		love.graphics.newQuad(48,48,16,16,64,64),
		
		
		-- quads for kinzo (why you would want to use this as a tileset is beyond me BUT YOU CAN)
		love.graphics.newQuad(0,0,16,16,96,16),
		love.graphics.newQuad(16,0,16,16,96,16),
		love.graphics.newQuad(32,0,16,16,96,16),
		love.graphics.newQuad(48,0,16,16,96,16),
		love.graphics.newQuad(64,0,16,16,96,16),
		love.graphics.newQuad(80,0,16,16,96,16),
	}
	
	editor_cur = cursor:new()
	cam = camera:new()
end


function love.update(dt)
	if mode == 1 then
		status_text = "<TILE_EDIT>"
	end
	
	if love.keyboard.isDown("a") then
		cam._x = cam._x - 5
	end
	
	if love.keyboard.isDown("d") then
		cam._x = cam._x + 5
	end
	
	if love.keyboard.isDown("w") then
		cam._y = cam._y - 5
	end
	
	if love.keyboard.isDown("s") then
		cam._y = cam._y + 5
	end
end


function love.draw()
	-- clear the set
	love.graphics.setColor(255,255,255)
	love.graphics.setBackgroundColor(64,64,64)
	-- set up the camera
	cam:place()
	-- draw the grid and stuffs
	love.graphics.setColor(255,255,255,128)
	love.graphics.draw(tile_grid,0,0)
	love.graphics.setColor(255,255,255)
	for i,v in pairs(blocks) do
		v:draw()
	end
	love.graphics.setColor(255,0,0)
	love.graphics.print(status_text,5,-35)
	love.graphics.print("X: "..editor_cur.x,5,-25)
	love.graphics.print("Y: "..editor_cur.y,5,-15)
	-- set the cursor to have 128 alpha and draw it
	love.graphics.setColor(255,255,255,128)
	editor_cur:draw()
	-- reset and let's get a move on
	love.graphics.setColor(255,255,255)
	cam:delete()
end



function love.keypressed(key)
	-- mode changing keys
	if key == "b" then
		mode = 1
	end
	
	-- autoclose key
	if key == "f8" then
		love.event.push("quit")
	end
	
	
	
	-- camera zoom YEAH LOL
	if key == "1" then
		cam:set_scale(1,1)
	end
	
	if key == "2" then
		cam:set_scale(0.5,0.5)
	end
	
	if key == "3" then
		cam:set_scale(2,2)
	end
	
	
	
	-- move the cursor BECAUSE THAT'S WHO I AM
	if key == "left" then
		editor_cur.x = editor_cur.x - 16
	end
	
	if key == "right" then
		editor_cur.x = editor_cur.x + 16
	end
	
	if key == "up" then
		editor_cur.y = editor_cur.y - 16
	end
	
	if key == "down" then
		editor_cur.y = editor_cur.y + 16
	end
	
	
	-- check for the t/y keys which change the tile indices
	if key == "t" then
		if editor_cur.quad - 1 == 0 then
			editor_cur.quad = 1
		else
			editor_cur.quad = editor_cur.quad - 1
		end
	end
	
	
	
	if key == "y" then
		if editor_cur.quad + 1 >= #quads then
			editor_cur.quad = #quads
		else
			editor_cur.quad = editor_cur.quad + 1
		end
	end
	
	
	if key == "e" then
		if editor_cur.index - 1 == 0 then
			editor_cur.index = 1
		else
			editor_cur.index = editor_cur.index - 1
		end
	end
	
	if key == "r" then
		if editor_cur.index + 1 >= #quads then
			editor_cur.index = #index
		else
			editor_cur.index = editor_cur.index + 1
		end
	end
	
	
	
	-- now let's allow tile placement (CAN I TRUST YOU GUYS NOT TO SPAM THIS)
	
	if key == "return" then
		status_text = "<PLACED_TILE>"
		table.insert(blocks,dummy:new(editor_cur.x,editor_cur.y,editor_cur.index,editor_cur.quad))
	end
	
	
	-- this gets rid of blocks (because we need this)
	if key == "p" then
		table.remove(blocks)
	end
	
	
	
	-- now let's allow exporting maps (because yes)
	if key == "v" then
		save_map()
	end
	
	
end


function love.keyreleased(key)
	if key == "return" then
		status_text = ""
	end
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end


function save_map()
	exports = exports + 1
	save_file = love.filesystem.newFile("output_"..exports.."("..math.floor(math.random(1,65535))..").nkz")
	save_file:open('w')
	save_file:write("--This map was created with the Haibutsu 2013.Editor\r\n\r\n\r\n")
	save_file:write("table.insert(objects[\"player\"],player:new(100,0))\r\n\r\n\r\n")
			for i,v in pairs(blocks) do
				save_file:write("table.insert(objects[\"entity\"],block:new("..v.x..","..v.y..","..v.tile..","..v.quad.."))		-- Block #"..i.."\r\n")
			end
	save_file:close()
	
	
end


function love.mousepressed(mx,my,mb)
	if mb == "l" then
		table.insert(blocks,dummy:new(math.floor(mx/16)+mx,math.floor(my/16)+my,editor_cur.index,editor_cur.quad))
	end
end